/* Host Application */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <unistd.h>

#include <e-hal.h>
#include "../shared.h"

#define FAIL(...) { fprintf(stderr, __VA_ARGS__); exit(1); }
#define SHM_OFFSET 0x01000000

/* helper functions */
void fixsudo(const char *filename);
void write_populations(block_t f[CORES_Y][CORES_X], int iter);
void write_density    (block_t f[CORES_Y][CORES_X], int iter);
void write_velocity   (block_t f[CORES_Y][CORES_X], int iter);
void convert_to_gif(void);
void convert_to_mp4(void);
void write_timers(times_t times[CORES_Y][CORES_X], uint32_t iter);

/* globals */
static shm_t    shm = { 0 };	/* local shm copy */
static uint32_t pollflag;

int main()
{
	char *filename = "bin/main.srec";

	/* remove old results */
	int dummy = system("rm -f ./tmp/i*.ppm ./tmp/anim.gif ./tmp/anim.mp4 populations.dat timers.dat");
	(void)dummy;

	e_epiphany_t dev;
	e_mem_t      mem;

	e_set_host_verbosity(H_D0);
	e_set_loader_verbosity(L_D0);

	/* initialize workgroup, allocate and clear shared memory */
	if(e_init(NULL) != E_OK)
		FAIL("Can't init!\n");
	e_reset_system();
	if(e_open(&dev, 0, 0, CORES_X, CORES_Y) != E_OK)
		FAIL("Can't open!\n");
	if(e_alloc(&mem, SHM_OFFSET, sizeof(shm_t)) != E_OK)
		FAIL("Can't alloc!\n");
	if(e_write(&mem, 0, 0, (off_t)0, &shm, sizeof(shm_t)) == E_ERR)
		FAIL("Can't clear shm!\n");

	/* load programs */
	printf("Starting cores:\n");
	for(int y = 0; y < CORES_Y; y++) {
		for(int x = 0; x < CORES_X; x++) {
			printf("(%02d,%02d) ", x, y);
			if(e_load(filename, &dev, x, y, E_TRUE) != E_OK)
				FAIL("Can't load!\n");
		}
		printf("\n");
	}

	/* ================================================================ */
	printf("Polling shared memory.\n");
	while(1) {
		while(1) {
			/* read polling flag */
			if(e_read(&mem, 0, 0, (off_t)0, &pollflag,
				sizeof(uint32_t)) == E_ERR)
					FAIL("Can't read pollflag!\n");

			/* wait for something */
			if(pollflag != POLL_BUSY) break;
		}

		/* finish if done */
		if(pollflag == POLL_DONE)
			break;

		/* read full shared memory */
		if(e_read(&mem, 0, 0, (off_t)0, &shm, sizeof(shm_t)) == E_ERR)
			FAIL("Can't read full shm!\n");

		/* reset pollflag */
		pollflag = 0;
		if(e_write(&mem, 0, 0, (off_t)0, &pollflag,
			sizeof(uint32_t)) == E_ERR) {
				FAIL("Can't reset pollflag!\n");
		}

		/* print iteration */
		printf("0x%08x\r", shm.iteration); fflush(stdout);

		/* write data */
		//write_populations(shm.lattice, shm.iteration);
		write_density(shm.lattice, shm.iteration);
		write_timers(shm.times, shm.iteration);
	}
	/* ================================================================ */

	if(e_free(&mem)  != E_OK) FAIL("Can't free!\n");
	if(e_close(&dev) != E_OK) FAIL("Can't close!\n");
	if(e_finalize()  != E_OK) FAIL("Can't finalize!\n");

	fixsudo("populations.dat");
	fixsudo("timers.dat");

	printf("\nProgram finished successfully.\n");
	printf("Convert ...\n");
	convert_to_gif();

	return(0);
}

