/* shared data types and external memory layout */
#ifndef _SHARED_H_
#define _SHARED_H_

#include <stdint.h>

/* ================================================================== */

/* number of cores */
#define CORES_X 4
#define CORES_Y 4

/* number of nodes per core */
#define NODES_X 10
#define NODES_Y 3
#define NODES_Z 10

/* ================================================================== */

/* preprocessor magic */
#define BUILD_BUG(c) do { ((void)sizeof(char[1 - 2*!!(c)])); } while(0);
#define USED __attribute__((used))
#undef PACKED
#define PACKED __attribute__((packed))
#undef ALIGN
#define ALIGN(X) __attribute__((aligned(X)))

/* some calculations */
#define CORES     (CORES_X * CORES_Y)
#define NODES     (NODES_X * NODES_Y * NODES_Z)
#define LATTICE_X (NODES_X * CORES_X)
#define LATTICE_Y (NODES_Y)
#define LATTICE_Z (NODES_Z * CORES_Y)

/* number of timer values */
#define TIMERS	12

/* pollflag values */
#define POLL_BUSY  0x00
#define POLL_READY 0x01
#define POLL_DONE  0x02

/* data types */
typedef float    FLOAT;
typedef FLOAT    node_t[19];
typedef node_t   block_t[NODES_Z][NODES_Y][NODES_X];
typedef uint32_t times_t[TIMERS];

/* shared memory structure */
typedef struct {
	uint32_t pollflag;
	uint32_t iteration;
	times_t  times[CORES_Y][CORES_X];
	block_t  lattice[CORES_Y][CORES_X];
} ALIGN(8) shm_t;

#endif /* _SHARED_H_ */

